==> netbeans_8.0.2+dfsg1.orig.tar.xz: netbeans-8.0.2/cnd.model.services/src/org/netbeans/modules/cnd/api/model/services/CsmIncludeResolver.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.cnd.api.model.services;

import org.netbeans.modules.cnd.api.model.CsmFile;
import org.netbeans.modules.cnd.api.model.CsmObject;
import org.netbeans.modules.cnd.utils.FSPath;
import org.openide.util.Lookup;

/**
 * Service that checks visibility of CSM object and finds include that makes CSM object visible
 * 
 * @author Nikolay Krasilnikov (nnnnnk@netbeans.org)
 */
public abstract class CsmIncludeResolver {

    /** A dummy resolver that never returns any results.
     */
    private static final CsmIncludeResolver EMPTY = new Empty();
    /** default instance */
    private static CsmIncludeResolver defaultResolver;
    
    protected CsmIncludeResolver() {
    }

    /** Static method to obtain the resolver.
     * @return the resolver
     */
    public static CsmIncludeResolver getDefault() {
        /*no need for sync synchronized access*/
        if (defaultResolver != null) {
            return defaultResolver;
        }
        defaultResolver = Lookup.getDefault().lookup(CsmIncludeResolver.class);
        return defaultResolver == null ? EMPTY : defaultResolver;
    }

    /**
     * Checks visibility of CSM object
     * 
     * @param currentFile - current file
     * @param item - CSM object (file as well)
     * @return - visibility of CSM object
     */
    public abstract boolean isObjectVisible(CsmFile currentFile, CsmObject item);
    
    /**
     * Finds best include directive for CSM object in format 
     * #include "file.h" or #include <file.h> or #include_next <file.h>
     * 
     * @param currentFile - current file
     * @param item - CSM object
     * @return - include directive string
     */
    public abstract String getIncludeDirective(CsmFile currentFile, CsmObject item);

    /**
     * Finds best include directive for CSM object in format 
     * #include "file.h"
     * 
     * @param path - current file path
     * @param item - CSM object
     * @return - include directive string
     */
    public abstract String getLocalIncludeDerectiveByFilePath(FSPath path, CsmObject item);

    //
    // Implementation of the default resolver
    //
    private static final class Empty extends CsmIncludeResolver {

        Empty() {
        }

        @Override
        public String getIncludeDirective(CsmFile currentFile, CsmObject item) {
            return "";
        }

        @Override
        public boolean isObjectVisible(CsmFile currentFile, CsmObject item) {
            return false;
        }

        @Override
        public String getLocalIncludeDerectiveByFilePath(FSPath path, CsmObject item) {
            return "";
        }
    }
}

==> netbeans_8.1+dfsg3.orig.tar.xz: netbeans-8.1/cnd.model.services/src/org/netbeans/modules/cnd/api/model/services/CsmIncludeResolver.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.cnd.api.model.services;

import org.netbeans.modules.cnd.api.model.CsmFile;
import org.netbeans.modules.cnd.api.model.CsmObject;
import org.netbeans.modules.cnd.utils.FSPath;
import org.openide.util.Lookup;

/**
 * Service that checks visibility of CSM object and finds include that makes CSM object visible
 * 
 * @author Nikolay Krasilnikov (nnnnnk@netbeans.org)
 */
public abstract class CsmIncludeResolver {

    /** A dummy resolver that never returns any results.
     */
    private static final CsmIncludeResolver EMPTY = new Empty();
    /** default instance */
    private static CsmIncludeResolver defaultResolver;
    
    protected CsmIncludeResolver() {
    }

    /** Static method to obtain the resolver.
     * @return the resolver
     */
    public static CsmIncludeResolver getDefault() {
        /*no need for sync synchronized access*/
        if (defaultResolver != null) {
            return defaultResolver;
        }
        defaultResolver = Lookup.getDefault().lookup(CsmIncludeResolver.class);
        return defaultResolver == null ? EMPTY : defaultResolver;
    }

    /**
     * Checks visibility of CSM object
     * 
     * @param currentFile - current file
     * @param item - CSM object (file as well)
     * @return - visibility of CSM object
     */
    public abstract boolean isObjectVisible(CsmFile currentFile, CsmObject item);

    /**
     * Get compilation unit close to header
     * 
     * @param file - header
     * @return - correspondent source file
     */
    public abstract CsmFile getCloseTopParentFile(CsmFile file);
    
    /**
     * Finds best include directive for CSM object in format 
     * #include "file.h" or #include <file.h> or #include_next <file.h>
     * 
     * @param currentFile - current file
     * @param item - CSM object
     * @return - include directive string
     */
    public abstract String getIncludeDirective(CsmFile currentFile, CsmObject item);

    /**
     * Finds best include directive for CSM object in format 
     * #include "file.h"
     * 
     * @param path - current file path
     * @param item - CSM object
     * @return - include directive string
     */
    public abstract String getLocalIncludeDerectiveByFilePath(FSPath path, CsmObject item);

    //
    // Implementation of the default resolver
    //
    private static final class Empty extends CsmIncludeResolver {

        Empty() {
        }

        @Override
        public String getIncludeDirective(CsmFile currentFile, CsmObject item) {
            return "";
        }

        @Override
        public boolean isObjectVisible(CsmFile currentFile, CsmObject item) {
            return false;
        }

        @Override
        public String getLocalIncludeDerectiveByFilePath(FSPath path, CsmObject item) {
            return "";
        }

        @Override
        public CsmFile getCloseTopParentFile(CsmFile currentFile) {
            return null;
        }
    }
}
